# -*- coding: UTF-8 -*-
'''
Created on 2017年11月21日
@author: lWX448714
'''
import os
import traceback
from tarfile import pwd, grp


def get_user_name(uid):
    users = pwd.getpwall()
    for user in users:
        if user.pw_uid == uid:
            return user.pw_name
    return None

def get_group_name(gid):
    groups = grp.getgrall()
    for group in groups:
        if str(group.gr_gid) == str(gid):
            return group.gr_name
    return None

def getMode(path):
    return oct(os.stat(path).st_mode)[-4:]

def getUAndGid(path):
    uid = os.stat(path).st_uid
    gid = os.stat(path).st_gid
    userName = get_user_name(uid)
    groupName = get_group_name(gid)
    return "%s:%s" % (userName, groupName)

def checkFile(cmode, smode, sowner, sgroup, filepath):
    msg = "NA"
    out = "NA"
    unsmode = ~int(smode, 8)
    if int(cmode, 8) & unsmode != 0:
            out = "fail"
    out = out.replace("\n", "")
    cowner = getUAndGid(filepath).replace("\n", "")
    if out == "fail" or cowner != "%s:%s" % (sowner, sgroup):
        msg = "%s owner:group %s mode:%s (should be %s:%s, %s or stricter)" % (filepath, cowner, cmode, sowner, sgroup, smode)
    return msg

def checkAUG(path):
    ret_code = 0
    results = []
    f = open(path, "r")
    try:
        lines = f.readlines()
        for line in lines:
            if len(results) >= 10:
                break
            if len(line) <= 2:
                continue
            msg = ""
            
            olines = line.split("\t")
            if len(olines) < 4:
                continue
            path = olines[0]
            sowner = olines[1]
            sgroup = olines[2]
            smode = olines[3]

            # check the files
            if "#" in path:
                part = path.replace("#", "")
                path = '/'.join(path.split("/")[:-1])

                for head, dirs, filenames in os.walk(path):
                    for fileName in filenames:
                        
                        if fileName == "":
                            continue
                        

                        filepath = os.path.join(head, fileName)

                        # consider the /a/name* and /a/b/#
                        if part not in filepath:
                            continue

                        if os.path.exists(filepath):
                            cmode = getMode(filepath).replace("\n", "")
                            msg = checkFile(cmode, smode , sowner, sgroup, filepath)
                            if msg == "NA":
                                continue
                            if msg != "" and len(results) <= 9:
                                results.append(msg)
            # check the dir 
            else:
                if "X" in path:
                    users = pwd.getpwall()
                    for user in users:
                        suser = user.pw_name
                        shome = user.pw_dir
                        suid = user.pw_uid
                    
                        if "/home" not in shome:
                            continue

                        sname = path.split("/")[-1]
                        sgroup = get_group_name(suid)

                        if "XXX" in sname:
                            fullpath = shome
                        else:
                            fullpath = os.path.join(shome, sname)

                        if os.path.exists(fullpath):

                            cmode = getMode(fullpath).replace("\n", "")
                            msg = checkFile(cmode, smode, suser, sgroup, fullpath)
                            if msg == "NA":
                                continue
                            if msg != "" and len(results) <= 9:
                                    results.append(msg)
                else:
                    if os.path.exists(path):
                        cmode = getMode(path).replace("\n", "")
                        msg = checkFile(cmode, smode, sowner, sgroup, path)

                        if msg == "NA":
                            continue
                        if msg != "" and len(results) <= 9:
                            results.append(msg)
    except:
        results.append(traceback.format_exc())
        ret_code = -1
    finally:
        f.close()
    
    return ret_code,results
    
def get_tags():
    return ['2','2.2','2.2.1']


def run_task(runcontext):
    ansible_params=runcontext.get_mod_params_instance()
    runcontext.set_mod_params_instance(ansible_params,"chdir",None)
    runcontext.set_mod_params_instance(ansible_params,"executable",None)
    module=runcontext.get_module_file("shell.py")
    rc,out=run_audit(runcontext,ansible_params,module)
    return rc,out
    
def run_audit(runcontext,ansible_params,module):
    curtemplate=runcontext.get_template()
    playbookdir = runcontext.get_parameter('playbookdir')
    
    ret_code = 0
    tmp_vars = {}
    tmp_vars["policyresult"] = "failed"
    tmp_vars["checkresult1"] = "failed"
    tmp_vars["systemvalue1"] = ""

    filePath = "%s/roles/hw/scripts/lfiles.txt" % playbookdir
    runcontext.log_debug(filePath)
    rc,results = checkAUG(filePath)
    if rc == 0:
        if len(results) == 0:
            tmp_vars["checkresult1"] = "pass"
            tmp_vars["policyresult"] = "pass"
        else:
            ret_code = -1
            tmp_vars["systemvalue1"] = '\n'.join(results)

        rc,out=curtemplate.render(runcontext.get_template_file('t_2_2_1.j2'),**tmp_vars)
        if rc == False:
            return (-2,"Failed to render template.%s" % out)
        return ret_code,out
    else:
        return (-2,results[0])
